/*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#ifndef _editor_
#define _editor_

#include "Config.h"

namespace AriaMaestosa {
	
class Track; //forward
class Sequence;
class GraphicalTrack;
class VolumeSlider;
class InstrumentChoice;

class Editor
{
	DECLARE_LEAK_CHECK();
	
    bool click_on_scrollbar; // if user is clicking on the scrollbar
    bool verticalScrolling; // is user is dragging the scroll thumb
    int lastDragY; // the Y position of the mouse during last drag event (to see how much mouse has moved between 2 events)
	
protected:
    float sb_position; // scrollbar position, 0 meaning at top and 1 at bottom
    bool scroll_up_arrow_pressed, scroll_down_arrow_pressed; // is the user holding down the arrows at the top and bottom of the scrollbar
    
    int from_y;
    int to_y;
    int width;
    int height;
    int barHeight;
	bool selecting;
	
public:
    InstrumentChoice* instrument;
        
    Editor(Track* track);
    virtual ~Editor();
    
    
    VolumeSlider* volumeSlider;
    
    Track* track;
    Sequence* sequence;
    GraphicalTrack* graphicalTrack;
    
	// Is it necessary to send frequent mouse held down events in current situation? this method tells you.
	bool areMouseHeldDownEventsNeeded();
	
    virtual void render(const int mousex_current=-1, const int mousey_current=-1, const int mousex_initial=-1, const int mousey_initial=-1, bool focus=false);
    
    void renderScrollbar();
    void updatePosition(const int from_y, const int to_y, const int width, const int height, const int barHeight);
    
    void drawVerticalMeasureLines(int from_y, int to_y);
    
    virtual void mouseDown(int x, int y);
    virtual void mouseDrag(int mousex_current, int mousey_current, int mousex_initial, int mousey_initial);
    virtual void mouseUp(int mousex_current, int mousey_current, int mousex_initial, int mousey_initial);
    virtual void rightClick(int x, int y);
    virtual void mouseHeldDown();
    virtual void mouseExited(int dragX_arg, int mousey_current, int XBeforeDrag_arg, int mousey_initial);
	
    const int getXEnd();
    const int getTrackYStart();
    const int getEditorYStart();
    const int getYEnd();
    const int getWidth();
    
    virtual int getYScrollInPixels();

    int snapMidiTickToGrid(int absolute_x);
};

const int getEditorXStart();
}

#endif
